/**
 * Client functions for user extension : temporary functions before server-side implementation
 */
import { Depositor } from '@sage/wh-master-data-api';
import { Site, User } from '@sage/wh-system-api';
import { getDefaultSiteFromModule, getUserProfile } from '@sage/wh-system/lib/client-functions/user-parameters-values';
import { DateValue } from '@sage/xtrem-date-time';
import * as ui from '@sage/xtrem-ui';

/**
 * get default site for the user for GEODE module
 * @param user
 * @returns default site for the user
 */
export function getDefaultSite(user: User): Site | null {
    return getUserProfile(user) ? getDefaultSiteFromModule(user, 2) : null;
}

/**
 * get string user code
 * @param user
 * @returns
 */
function _getUserCode(user: User | string): string {
    return typeof user === 'string' ? user?.toUpperCase() : user?.code;
}

/**
 * get site for the user for GEODE module
 * @param user
 * @returns default site for the user
 */

export async function getByUserCurrentSiteCode(pageInstance: any, userCode: string): Promise<string | null> {
    return (await _readByUserParameterValueString(pageInstance, 'FCY', _getUserCode(userCode))) ?? null;
}

/**
 * get depositor for the user for GEODE module
 * @param userCode
 * @returns default site for the user
 */
export async function getByUserCurrentDepositorCode(pageInstance: any, userCode: string): Promise<string | null> {
    return (await _readByUserParameterValueString(pageInstance, 'FCY', _getUserCode(userCode)))
        ? ((await _readByUserParameterValueString(pageInstance, 'DEP', _getUserCode(userCode))) ?? null)
        : null;
}

/**
 * Check if you can enter the site
 * @param pageInstance current page instance
 * @param userCode
 * @returns
 */
export async function getIsByUserAllowedSiteChange(pageInstance: any, userCode: string): Promise<boolean> {
    return !!(await _readByUserParameterValueBoolean(pageInstance, 'MULFCY', _getUserCode(userCode)));
}

/**
 * @param pageInstance current page instance
 * Check if you can enter the depositor
 * @param userCode
 * @returns
 */
export async function getIsByUserAllowedDepositorChange(pageInstance: any, userCode: string): Promise<boolean> {
    return !!(await _readByUserParameterValueBoolean(pageInstance, 'MULDEP', _getUserCode(userCode)));
}

/**
 * Read record section
 */

/**
 * Read site from given site
 * Only instance with property code
 * @param pageInstance
 * @param siteCode
 * @param depositorCode
 * @returns site when exist
 */
export async function readSite(pageInstance: any, siteCode: string): Promise<Site | null> {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-system/Site')
            .read(
                {
                    code: true,
                },
                `${siteCode?.toUpperCase()}`,
            )
            .execute()) as Site | null;
    } catch (error) {
        return null;
    }
}

/**
 * Read depositor from given site and depositor codes
 * Only instance with property site and code
 * @param pageInstance
 * @param siteCode
 * @param depositorCode
 * @returns depositor when exist
 */
export async function readDepositor(
    pageInstance: any,
    siteCode: string,
    depositorCode: string,
): Promise<Depositor | null> {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-master-data/Depositor')
            .read(
                {
                    site: {
                        code: true,
                    },
                    code: true,
                },
                `${siteCode?.toUpperCase()}|${depositorCode?.toUpperCase()}`,
            )
            .execute()) as Depositor | null;
    } catch (error) {
        return null;
    }
}

/**
 *  Read user from given user code for print destinations and site depositor
 * @param pageInstance
 * @param userCode
 * @returns
 */
export async function readUser(pageInstance: any, userCode?: string): Promise<User | null> {
    try {
        return (await pageInstance.$.graph
            .node('@sage/wh-system/User')
            .read(
                {
                    code: true,
                    name: true,
                    functionProfile: {
                        profile: true,
                        userProfileDefaultSites: {
                            query: {
                                edges: {
                                    node: {
                                        site: {
                                            code: true,
                                        },
                                    },
                                },
                            },
                        },
                    },
                    profession: {
                        profession: true,
                        functionProfile: {
                            profile: true,
                            userProfileDefaultSites: {
                                query: {
                                    edges: {
                                        node: {
                                            site: {
                                                code: true,
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                    userDestinations: {
                        query: {
                            edges: {
                                node: {
                                    code: true,
                                    destination: {
                                        code: true,
                                        description: true,
                                        destination: true,
                                    },
                                },
                            },
                        },
                    },
                    currentDepositorCode: true,
                    currentSiteCode: true,
                    defaultSiteCode: true,
                    isAllowedDepositorChange: true,
                    isAllowedSiteChange: true,
                },
                `${userCode?.toUpperCase() ?? pageInstance.$.userCode?.toUpperCase()}`,
            )
            .execute()) as User | null;
    } catch (error) {
        return null;
    }
}

/**
 * General section for current user
 */

/**
 * Read the value of a string parameter
 * @param pageInstance
 * @param parameter
 * @returns
 */
async function _readParameterValueString(pageInstance: ui.Page, parameter: string): Promise<string | undefined> {
    try {
        return (
            await pageInstance.$.graph
                .node('@sage/wh-system/UserParameterValue')
                .mutations.readParameterValueString(
                    { value: true },
                    {
                        parameters: {
                            code: parameter,
                        },
                    },
                )
                .execute()
        )?.value;
    } catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}

/**
 *  Read the value of an integer parameter
 * @param pageInstance
 * @param parameter
 * @returns
 */
async function _readParameterValueInteger(pageInstance: ui.Page, parameter: string): Promise<number | undefined> {
    try {
        return (
            await pageInstance.$.graph
                .node('@sage/wh-system/UserParameterValue')
                .mutations.readParameterValueInteger(
                    { value: true },
                    {
                        parameters: {
                            code: parameter,
                        },
                    },
                )
                .execute()
        )?.value;
    } catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}

/**
 *  Read the value of a date parameter
 * @param pageInstance
 * @param parameter
 * @returns
 */
async function _readParameterValueDate(pageInstance: ui.Page, parameter: string): Promise<DateValue | undefined> {
    try {
        return (
            await pageInstance.$.graph
                .node('@sage/wh-system/UserParameterValue')
                .mutations.readParameterValueDate(
                    { value: true },
                    {
                        parameters: {
                            code: parameter,
                        },
                    },
                )
                .execute()
        )?.value;
    } catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}

/**
 * Read a parameter value as a boolean
 * @param pageInstance
 * @param parameter
 * @returns
 */
async function _readParameterValueBoolean(
    pageInstance: ui.Page,
    parameter: string,
    user: string,
): Promise<boolean | undefined> {
    const _value = await _readParameterValueInteger(pageInstance, parameter);
    return _value ? _value === 2 : undefined;
}

/**
 * Sticker site depositor section (need administrator rights for using user)
 */

/**
 * Read the value of a string parameter for given user
 * @param pageInstance
 * @param parameter
 * @param user
 * @returns
 */
async function _readByUserParameterValueString(
    pageInstance: ui.Page,
    parameter: string,
    user: string,
): Promise<string | undefined> {
    try {
        return (
            await pageInstance.$.graph
                .node('@sage/wh-system/UserParameterValue')
                .mutations.readByUserParameterValueString(
                    { value: true },
                    {
                        parameters: {
                            user: user,
                            code: parameter,
                        },
                    },
                )
                .execute()
        )?.value;
    } catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}

/**
 *  Read the value of an integer parameter for given user
 * @param pageInstance
 * @param parameter
 * @param user
 * @returns
 */
async function _readByUserParameterValueInteger(
    pageInstance: ui.Page,
    parameter: string,
    user: string,
): Promise<number | undefined> {
    try {
        return (
            await pageInstance.$.graph
                .node('@sage/wh-system/UserParameterValue')
                .mutations.readByUserParameterValueInteger(
                    { value: true },
                    {
                        parameters: {
                            user: user,
                            code: parameter,
                        },
                    },
                )
                .execute()
        )?.value;
    } catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}

/**
 *  Read the value of a date parameter for given user
 * @param pageInstance
 * @param parameter
 * @param user
 * @returns
 */
async function _readByUserParameterValueDate(
    pageInstance: ui.Page,
    parameter: string,
    user: string,
): Promise<DateValue | undefined> {
    try {
        return (
            await pageInstance.$.graph
                .node('@sage/wh-system/UserParameterValue')
                .mutations.readByUserParameterValueDate(
                    { value: true },
                    {
                        parameters: {
                            user: user,
                            code: parameter,
                        },
                    },
                )
                .execute()
        )?.value;
    } catch (e) {
        ui.console.error(`Error reading parameter value ${parameter} : `, e.message);
        return undefined;
    }
}

/**
 * Read a parameter value as a boolean for given user
 * @param pageInstance
 * @param parameter
 * @param user
 * @returns
 */
async function _readByUserParameterValueBoolean(
    pageInstance: ui.Page,
    parameter: string,
    user: string,
): Promise<boolean | undefined> {
    const _value = await _readByUserParameterValueInteger(pageInstance, parameter, user);
    return _value ? _value === 2 : undefined;
}
